{******************************************************************************}
{*    "DirectX.    Delphi".                  *}
{*  BHV. www.bhv.ru                                               *}
{*  .. softgl@chat.ru                                                *}
{******************************************************************************}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  AppEvnts,
  DirectXGraphics, DXGUtils;

type
  TfrmD3D = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    procedure FormCreate(Sender: TObject);
    procedure ApplicationEvents1Minimize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ApplicationEvents1Restore(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FD3D            : IDIRECT3D8;
    FD3DDevice      : IDIRECT3DDEVICE8;
    FD3DVB          : IDIRECT3DVERTEXBUFFER8;
    FActive         : BOOL;
    ThisTickCount   : DWORD;
    LastTickCount   : DWORD;
    function Render : HRESULT;
    function InitD3D : HRESULT;
    function InitVB : HRESULT;
    procedure DrawScene;
    procedure Cleanup;
    procedure SetupLights;
    procedure ErrorOut (const Caption : PChar; const hError : HRESULT);
  public
    { Public declarations }
  end;

type
  TCUSTOMVERTEX = packed record
    X, Y, Z : Single;
    nX, nY, nZ : Single;
    U, V : Single;
  end;

const
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1;

var
  frmD3D: TfrmD3D;
  Frames : Integer = 0;

  Light     : TD3DLight8;

  Angle : Single = 0;

const
{$I torus.pas}
{$I sphere.pas}

implementation

{$R *.DFM}

procedure TfrmD3D.Cleanup;
begin
  if Assigned (FD3DVB) then begin
     FD3DVB._Release;
     FD3DVB := nil;
  end;

  if Assigned (FD3DDevice) then begin
     FD3DDevice._Release;
     FD3DDevice := nil;
  end;

  if Assigned (FD3D) then begin
     FD3D._Release;
     FD3D := nil;
  end;
end;

procedure TfrmD3D.ErrorOut (const Caption : PChar; const hError : HRESULT);
begin
  FActive := False;
  Cleanup;
  MessageBox (Handle, PChar(DXGErrorString (hError)), Caption, 0);
end;

function TfrmD3D.InitD3D : HRESULT;
var
  d3ddm : TD3DDISPLAYMODE;
  d3dpp : TD3DPRESENT_PARAMETERS;
  hRet : HRESULT;
begin
  FD3DDevice := nil;
  FD3DVB := nil;
  FD3D := nil;

  FD3D := Direct3DCreate8(D3D_SDK_VERSION);
  if FD3D = nil then begin
     Result := E_FAIL;
     Exit;
  end;

  hRet := FD3D.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, d3ddm);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  ZeroMemory(@d3dpp, SizeOf(d3dpp));
  with d3dpp do begin
       Windowed := True;
       SwapEffect := D3DSWAPEFFECT_DISCARD;
       BackBufferFormat := d3ddm.Format;
       EnableAutoDepthStencil := True;
       AutoDepthStencilFormat := D3DFMT_D16;
  end;

  Result := FD3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, Handle,
                              D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                              d3dpp, FD3DDevice);
end;

function TfrmD3D.Render : HRESULT;
var
  hRet : HRESULT;
begin
  if FD3DDevice = nil then begin
     Result := E_FAIL;
     Exit;
  end;

  hRet := FD3DDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                           $000000FF, 1.0, 0);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.BeginScene;
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  with FD3DDevice do begin
       SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
       SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
       SetRenderState(D3DRS_NORMALIZENORMALS, Dword (True));
  end;

  DrawScene;

  hRet := FD3DDevice.EndScene;
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  Result := FD3DDevice.Present(nil, nil, 0, nil);
end;

procedure TfrmD3D.FormCreate(Sender: TObject);
var
  hRet : HRESULT;
  matView, matProj : TD3DMatrix;
begin
  hRet := InitD3D;
  if Failed (hRet) then ErrorOut ('InitD3D', hRet);
  hRet := InitVB;
  if Failed (hRet) then ErrorOut ('InitVertex', hRet);

  SetupLights;

  SetViewMatrix(matView, D3DVector(0.0, 1.0, 1.0), D3DVector(0, 0, 0), D3DVector(0, 1, 0));
  FD3DDevice.SetTransform(D3DTS_VIEW, matView);

  SetProjectionMatrix(matProj, 1, 1, 0.1, 10);
  FD3DDevice.SetTransform(D3DTS_PROJECTION, matProj);
end;

procedure TfrmD3D.ApplicationEvents1Minimize(Sender: TObject);
begin
  FActive := False;
end;

procedure TfrmD3D.FormActivate(Sender: TObject);
begin
  LastTickCount := GetTickCount;
  FActive := True;
end;

procedure TfrmD3D.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
var
  hRet : HRESULT;
begin
  if FActive then begin
     Inc (Frames);
     hRet := Render;
     if FAILED(hRet) then begin
        FActive := False;
        ErrorOut ('Render', hRet);
        Exit;
     end;

     ThisTickCount := GetTickCount;
     if ThisTickCount - LastTickCount > 15 then begin
        Caption := 'FPS = ' + Format('%6.2f',
                                 [frames * 1000 / (ThisTickCount - LastTickCount)]);
        Frames := 0;

        Angle := Angle + 0.05;
        if Angle > 2 * Pi then Angle := Angle - 2 * Pi;

        LastTickCount := GetTickCount;
     end;
  end;

  Done := False;
end;

procedure TfrmD3D.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then Close;
end;

procedure TfrmD3D.ApplicationEvents1Restore(Sender: TObject);
begin
  FActive := True;
end;

procedure TfrmD3D.FormDestroy(Sender: TObject);
begin
  Cleanup
end;

function TfrmD3D.InitVB : HRESULT;
var
  hRet : HRESULT;
  Vertices : ^TCustomVertex;
  i, j : Integer;
  vi : Integer;
  ni : Integer;
begin
  hRet := FD3DDevice.CreateVertexBuffer((864 + 1200) * 3 * SizeOf(TCustomVertex), 0,
                                        D3DFVF_CUSTOMVERTEX,
                                        D3DPOOL_DEFAULT, FD3DVB);
  if Failed(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.SetStreamSource(0, FD3DVB, SizeOf(TCustomVertex));
  if Failed(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.SetVertexShader(D3DFVF_CUSTOMVERTEX);
  if Failed(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DVB.Lock(0, (864 + 1200) * 3 * SizeOf(TCustomVertex), PByte(Vertices), 0);
  if Failed(hRet) then begin
     Result := hRet;
     Exit;
  end;

  for i := 0 to 863 do
      for j := 0 to 2 do begin
      vi := Torusface_indicies[i][j];
      ni := Torusface_indicies[i][j+3];

         Vertices.X := TorusVertices[vi][0];
         Vertices.Y := TorusVertices[vi][1];
         Vertices.Z := TorusVertices[vi][2];

         Vertices.nX := Torusnormals[ni][0];
         Vertices.nY := Torusnormals[ni][1];
         Vertices.nZ := Torusnormals[ni][2];

         Inc(Vertices);
  end;

  for i := 0 to 1199 do
      for j := 0 to 2 do begin
      vi := Sphereface_indicies[i][j];
      ni := Sphereface_indicies[i][j+3];

         Vertices.X := SphereVertices[vi][0];
         Vertices.Y := SphereVertices[vi][1];
         Vertices.Z := SphereVertices[vi][2];

         Vertices.nX := Spherenormals[ni][0];
         Vertices.nY := Spherenormals[ni][1];
         Vertices.nZ := Spherenormals[ni][2];

         Inc(Vertices);
  end;

  Result := FD3DVB.Unlock;
end;

procedure TfrmD3D.DrawScene;
var
  matTranslate, matScale : TD3DMatrix;
begin
  Light.Position := D3DVector(0.0, cos (Angle) * 2, 0.0);
  with FD3DDevice do begin
       SetTransform(D3DTS_WORLD, IdentityMatrix);
       SetLight(0, Light);
       DrawPrimitive(D3DPT_TRIANGLELIST, 0, 864);
  end;

  Light.Position := D3DVector(0.0, 0.0, 0.0);
  SetTranslateMatrix(matTranslate, 0.0, cos (Angle) * 2, 0.0);
  SetScaleMatrix(matScale, 0.1, 0.1, 0.1);
  with FD3DDevice do begin
       SetTransform(D3DTS_WORLD, MatrixMul(matScale, matTranslate));
       SetLight(0, Light);
       DrawPrimitive(D3DPT_TRIANGLELIST, 864 * 3, 1200);
  end;
end;

procedure TfrmD3D.SetupLights;
var
  Material  : TD3DMaterial8;
begin
  Material := InitMaterial(1, 1, 0, 0);
  FD3DDevice.SetMaterial(Material);

  ZeroMemory(@Light, SizeOf(Light));
  with Light do begin
       _Type := D3DLIGHT_POINT;
       Diffuse.R := 1.0;
       Diffuse.G := 1.0;
       Diffuse.B := 1.0;
       Specular := Diffuse;
       Ambient := Diffuse;
       Position := D3DVector(0.0, 0.0, 0.0);
       Attenuation0 := 1.0;
       Attenuation1 := 1.0;
       Attenuation2 := 1.0;
       Range := 2.5;
  end;

  FD3DDevice.SetLight(0, Light);

  FD3DDevice.LightEnable(0, True);
end;

end.
